#include "pch.h"

#include "hook_thread.h"

static HANDLE           _hThread;
static HWND             _hWnd;
static volatile LONG    _iNotified;

static void ProcessKeyboardInput(const KBDLLHOOKSTRUCT * pStruct)
{
    if (
        !(pStruct->flags & LLKHF_UP) &&
        pStruct->vkCode == VK_CAPITAL &&
        !_InterlockedExchange(&_iNotified, TRUE))
        PostMessageW(_hWnd, WM_CAPS_LOCK_DOWN, 0, 0);
}

static LRESULT CALLBACK
    LowLevelKeyboardProc(int nCode, WPARAM wparam, LPARAM lparam)
{
    LRESULT lresult;
    
    lresult = CallNextHookEx(NULL, nCode, wparam, lparam);
    ProcessKeyboardInput((const KBDLLHOOKSTRUCT *)lparam);
    return lresult;
}

static void MessageLoop()
{
    MSG msg;
    
    while (GetMessageW(&msg, NULL, 0, 0));
}

static DECLSPEC_NORETURN void WINAPI ThreadProc(LPVOID lpParameter)
{
    const HWND hWnd = (HWND)lpParameter;
    _hWnd = hWnd;
    // On Windows XP the hmod parameter cannot be NULL.
    if (
        SetWindowsHookExW(
        WH_KEYBOARD_LL,
        LowLevelKeyboardProc,
        GetModuleHandleW(NULL),
        0))
        MessageLoop();
    SendMessage(hWnd, WM_CLOSE, 0, 0);
}

void ResetCapsLockDownStatus()
{
    _iNotified = FALSE;
}

BOOL StartHookThread(HWND hWnd)
{
    HANDLE hThread;

    hThread =
        CreateThread(
        NULL,
        0,
        (LPTHREAD_START_ROUTINE)ThreadProc,
        (LPVOID)hWnd,
        CREATE_SUSPENDED,
        NULL
        );
    if (hThread == NULL) return FALSE;
    _hThread = hThread;
    SetThreadPriority(hThread, THREAD_PRIORITY_TIME_CRITICAL);
    ResumeThread(hThread);
    return TRUE;
}

void TerminateHookThread()
{
    TerminateThread(_hThread, 0);
}
