#include "pch.h"

#include "hook_thread.h"
#include "notifier.h"

static LRESULT CALLBACK
    WndProc(HWND hWnd, UINT message, WPARAM wparam, LPARAM lparam)
{
    switch (message)
    {
    case WM_DESTROY:
        TerminateHookThread();
        PostQuitMessage(0);
        break;
    case WM_CAPS_LOCK_DOWN:
        ShowCapsLockNotifier();
        ResetCapsLockDownStatus();
        break;
    default:
        return DefWindowProc(hWnd, message, wparam, lparam);
    }
    return 0;
}

static HWND CreateUniqueWindow()
{
    HANDLE hMutex;
    HWND hWnd;

    const PCWSTR szClassName = L"Message";
    const PCWSTR szWindowName = L"Caps Lock Notifier";
    hMutex = CreateMutexW(NULL, FALSE, L"Caps Lock Notifier");
    WaitForSingleObject(hMutex, INFINITE);
    hWnd =
        FindWindowW(szClassName, szWindowName) != NULL
        ?
        NULL
        :
        CreateWindowExW(
        0,
        szClassName,
        szWindowName,
        0,
        0,
        0,
        0,
        0,
        HWND_MESSAGE,
        NULL,
        NULL,
        NULL
        );
    ReleaseMutex(hMutex);
    CloseHandle(hMutex);
    return hWnd;
}

static BOOL Initialize()
{
    HWND hWnd;

    hWnd = CreateUniqueWindow();
    if (hWnd != NULL && InitCapsLockNotifier())
    {
        // Must set the window procedure before the hook thread starts.
        SetWindowLongPtrW(hWnd, GWL_WNDPROC, (LONG_PTR)WndProc);
        if (StartHookThread(hWnd)) return TRUE;
    }
    return FALSE;
}

static UINT MessageLoop()
{
    MSG msg;
    
    while (GetMessageW(&msg, NULL, 0, 0)) DispatchMessageW(&msg);
    return (UINT)msg.wParam;
}

DECLSPEC_NORETURN void APIENTRY EntryPoint()
{
    ExitProcess(Initialize() ? MessageLoop() : 0);
}
